var amqp = require('amqplib/callback_api');

module.exports.TopicHandler = function (url) {
    this.url = url;
    this.listen = listen;
    this.publish = publish;

    function listen(ex, key, cb) {
        amqp.connect(this.url, function (err, conn) {
            conn.createChannel(function (err, ch) {

                ch.assertExchange(ex, 'topic', { durable: false });

                ch.assertQueue('', { exclusive: true }, function (err, q) {
                    ch.bindQueue(q.queue, ex, key);
                    console.log("Listening from topic:%s key:%s queue:%s", ex, key, q.queue);
                    ch.consume(q.queue, function (msg) {
                        console.log("Receiving from topic:%s key:%s: '%s'", ex, key, msg.content.toString());
                        let data = JSON.parse(msg.content.toString())
                        cb(data);
                    }, { noAck: true });
                });
            });
        });
    }

    function publish(ex, key, msg, cb) {
        amqp.connect(this.conn, function (err, conn) {
            conn.createChannel(function (err, ch) {
                ch.assertExchange(ex, 'topic', { durable: false });
                console.log("Sending to topic:%s key:%s: '%s'", ex, key, msg);
                ch.publish(ex, key, new Buffer(msg));
            });
            setTimeout(function () {
                conn.close();
                cb();
            }, 500);
        })

    }
}

module.exports.QueueHandler = function (url) {
    this.url = url;
    this.listen = listen;
    this.publish = publish;

    function listen(q, cb) {
        amqp.connect(this.url, function (err, conn) {
            conn.createChannel(function (err, ch) {
                ch.assertQueue(q, { durable: true });
                ch.prefetch(1);
                console.log("Listening from queue:%s", q);
                ch.consume(q, function (msg) {
                    console.log("Receiving from queue:%s: '%s'", q, msg.content.toString());
                    let data = JSON.parse(msg.content.toString())
                    cb(data, function() {
                        ch.ack(msg);
                    })
                }, { noAck: false });
            });
        });
    }

    function publish(q, msg, cb) {
        amqp.connect(this.url, function (err, conn) {
            conn.createChannel(function (err, ch) {
                ch.assertQueue(q, { durable: true });
                ch.sendToQueue(q, new Buffer(msg), { persistent: true });
                console.log("Sending to queue:%s: '%s'", q, msg);
            });
            setTimeout(function () {
                conn.close();
                cb();
            }, 500);
        });
    }


}